﻿using PComm;
using PObject;
using PUtility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ClientForm
{
	public partial class MainForm : Form
	{
		private readonly string SERVER_IP = "124.56.13.173";

		private PClient heartBeatClient;
		private PClient client;

		private System.Timers.Timer heartBeatTimer;
		private System.Timers.Timer GarbageTimer;

		public MainForm()
		{
			InitializeComponent();
			InitInstance();
		}

		private void InitInstance()
		{
			string clientId = PUtil.GetRandomString(5);
			string heartBeatClientId = clientId + "_hb";

			heartBeatClient = new PClient(SERVER_IP, 37778, heartBeatClientId);
			heartBeatClient.OnReceived += HeartBeatClient_OnReceived;

			client = new PClient(SERVER_IP, 37777, clientId);
			client.OnReceived += Client_OnReceived;

			heartBeatTimer = new System.Timers.Timer();
			heartBeatTimer.Interval = 1000 * 10;
			heartBeatTimer.Elapsed += HeartBeatTimer_Elapsed;
			heartBeatTimer.Start();

			GarbageTimer = new System.Timers.Timer();
			GarbageTimer.Interval = 1000 * 10;
			GarbageTimer.Elapsed += GarbageTimer_Elapsed;
			GarbageTimer.Start();
		}

		private void WriteLog(string log)
		{
			if (this.InvokeRequired)
			{
				Invoke((MethodInvoker)delegate () { WriteLog(log); });
			}
			else
			{
				logConsole.AppendText($"[{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}] {log}" + Environment.NewLine);
				//PUtil.ScrollToEnd(logConsole);
			}
		}

		private void HeartBeatTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			heartBeatClient.Send(PServer.HB_CHECK);
		}

		private void HeartBeatClient_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			if (dataType != PDataType.SimpleString)
			{
				WriteLog("ERROR!!! WRONG DATA TYPE FOR HEART BEAT!!!");
				return;
			}

			string msg = Encoding.UTF8.GetString(data, 0, data.Length);
			if (msg != PServer.HB_CHECK)
			{
				WriteLog("WARNING!!! BAD HEARTBEAT!!!");
				client.Reconnect();
				heartBeatClient.Reconnect();
			}
				
		}

		private void Client_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			throw new NotImplementedException();
		}

		private void GarbageTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			GC.Collect();
		}

		private void stringButton_Click(object sender, EventArgs e)
		{
			client.Send("String send test!");
		}

		private void objectButton_Click(object sender, EventArgs e)
		{
			Random rand = new Random();
			Person person = new Person()
			{
				Name = PUtil.GetRandomString(5),
				Age = rand.Next(20, 100)
			};

			byte[] data = PSerializer.Serialize(person);
			client.Send(PDataType.PersonObject, data);
		}

		private void imageButton_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Filter = "JPG File|*.jpg|PNG File|*.png|All File|*.*";
			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			Image img = Image.FromFile(dlg.FileName);
			byte[] data = PUtil.ImageToBytes(img);
			client.Send(PDataType.ShowImage, data);

			GC.Collect();
		}

		private void fileButton_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Filter = "All File|*.*";
			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			string filePath = dlg.FileName;
			string fileName = dlg.SafeFileName;

			PFileData fileData = new PFileData()
			{
				FileName = fileName,
				Data = PUtil.FileToBytes(filePath)
			};

			if (fileData.Data.Length >= 1000000000)
			{
				MessageBox.Show("1GB를 초과하는 데이터는 전송할 수 없습니다.");

				GC.Collect();
				return;
			}


			byte[] data = PSerializer.Serialize(fileData);
			client.Send(PDataType.SaveFile, data);

			GC.Collect();
		}
	}
}
